@@ -28,7 +28,7 @@ import (
 	"k8s.io/kubernetes/test/e2e_node/system"
 	"k8s.io/kubernetes/test/utils"
 
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 )
 
 var buildDependencies = flag.Bool("build-dependencies", true, "If true, build all dependencies.")
@@ -36,12 +36,13 @@ var ginkgoFlags = flag.String("ginkgo-flags", "", "Space-separated list of argum
 var testFlags = flag.String("test-flags", "", "Space-separated list of arguments to pass to node e2e test.")
 var systemSpecName = flag.String("system-spec-name", "", fmt.Sprintf("The name of the system spec used for validating the image in the node conformance test. The specs are at %s. If unspecified, the default built-in spec (system.DefaultSpec) will be used.", system.SystemSpecPath))
 var extraEnvs = flag.String("extra-envs", "", "The extra environment variables needed for node e2e tests. Format: a list of key=value pairs, e.g., env1=val1,env2=val2")
+var runtimeConfig = flag.String("runtime-config", "", "The runtime configuration for the API server on the node e2e tests. Format: a list of key=value pairs, e.g., env1=val1,env2=val2")
 
 func main() {
 	klog.InitFlags(nil)
 	flag.Parse()
 
-	// Build dependencies - ginkgo, kubelet and apiserver.
+	// Build dependencies - ginkgo, kubelet, e2e_node.test, and mounter.
 	if *buildDependencies {
 		if err := builder.BuildGo(); err != nil {
 			klog.Fatalf("Failed to build the dependencies: %v", err)
@@ -57,7 +58,7 @@ func main() {
 	ginkgo := filepath.Join(outputDir, "ginkgo")
 	test := filepath.Join(outputDir, "e2e_node.test")
 
-	args := []string{*ginkgoFlags, test, "--", *testFlags}
+	args := []string{*ginkgoFlags, test, "--", *testFlags, fmt.Sprintf("--runtime-config=%s", *runtimeConfig)}
 	if *systemSpecName != "" {
 		rootDir, err := utils.GetK8sRootDir()
 		if err != nil {
